@@ -12,6 +12,7 @@
 
 #include <aws/io/event_loop.h>
 #include <aws/io/host_resolver.h>
+#include <aws/io/private/event_loop_impl.h>
 #include <aws/io/socket.h>
 
 #ifdef _MSC_VER
@@ -447,7 +448,7 @@ static int s_test_socket_with_bind_to_interface(struct aws_allocator *allocator,
 #else
     strncpy(options.network_interface_name, "lo", AWS_NETWORK_INTERFACE_NAME_MAX);
 #endif
-    struct aws_socket_endpoint endpoint = {.address = "127.0.0.1", .port = 8127};
+    struct aws_socket_endpoint endpoint = {.address = "127.0.0.1", .port = 8128};
     if (s_test_socket(allocator, &options, &endpoint)) {
 #if !defined(AWS_OS_APPLE) && !defined(AWS_OS_LINUX)
         if (aws_last_error() == AWS_ERROR_PLATFORM_NOT_SUPPORTED) {
@@ -460,7 +461,7 @@ static int s_test_socket_with_bind_to_interface(struct aws_allocator *allocator,
     options.domain = AWS_SOCKET_IPV4;
     ASSERT_SUCCESS(s_test_socket(allocator, &options, &endpoint));
 
-    struct aws_socket_endpoint endpoint_ipv6 = {.address = "::1", .port = 1024};
+    struct aws_socket_endpoint endpoint_ipv6 = {.address = "::1", .port = 8129};
     options.type = AWS_SOCKET_STREAM;
     options.domain = AWS_SOCKET_IPV6;
     if (s_test_socket(allocator, &options, &endpoint_ipv6)) {
@@ -496,6 +497,20 @@ static int s_test_socket_with_bind_to_invalid_interface(struct aws_allocator *al
 }
 AWS_TEST_CASE(test_socket_with_bind_to_invalid_interface, s_test_socket_with_bind_to_invalid_interface)
 
+static int s_test_is_network_interface_name_valid(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    (void)allocator;
+
+    ASSERT_FALSE(aws_is_network_interface_name_valid("invalid_name"));
+#if defined(AWS_OS_LINUX)
+    ASSERT_TRUE(aws_is_network_interface_name_valid("lo"));
+#elif !defined(AWS_OS_WINDOWS)
+    ASSERT_TRUE(aws_is_network_interface_name_valid("lo0"));
+#endif
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(test_is_network_interface_name_valid, s_test_is_network_interface_name_valid)
+
 #if defined(USE_VSOCK)
 static int s_test_vsock_loopback_socket_communication(struct aws_allocator *allocator, void *ctx) {
 /* Without vsock loopback it's difficult to test vsock functionality.
@@ -610,7 +625,10 @@ static int s_test_connect_timeout(struct aws_allocator *allocator, void *ctx) {
 
     aws_io_library_init(allocator);
 
-    struct aws_event_loop_group *el_group = aws_event_loop_group_new_default(allocator, 1, NULL);
+    struct aws_event_loop_group_options elg_options = {
+        .loop_count = 1,
+    };
+    struct aws_event_loop_group *el_group = aws_event_loop_group_new(allocator, &elg_options);
     struct aws_event_loop *event_loop = aws_event_loop_group_get_next_loop(el_group);
     ASSERT_NOT_NULL(event_loop, "Event loop creation failed with error: %s", aws_error_debug_str(aws_last_error()));
 
@@ -690,7 +708,10 @@ static int s_test_connect_timeout_cancelation(struct aws_allocator *allocator, v
 
     aws_io_library_init(allocator);
 
-    struct aws_event_loop_group *el_group = aws_event_loop_group_new_default(allocator, 1, NULL);
+    struct aws_event_loop_group_options elg_options = {
+        .loop_count = 1,
+    };
+    struct aws_event_loop_group *el_group = aws_event_loop_group_new(allocator, &elg_options);
     struct aws_event_loop *event_loop = aws_event_loop_group_get_next_loop(el_group);
     ASSERT_NOT_NULL(event_loop, "Event loop creation failed with error: %s", aws_error_debug_str(aws_last_error()));
 
@@ -1133,7 +1154,10 @@ static int s_cleanup_before_connect_or_timeout_doesnt_explode(struct aws_allocat
 
     aws_io_library_init(allocator);
 
-    struct aws_event_loop_group *el_group = aws_event_loop_group_new_default(allocator, 1, NULL);
+    struct aws_event_loop_group_options elg_options = {
+        .loop_count = 1,
+    };
+    struct aws_event_loop_group *el_group = aws_event_loop_group_new(allocator, &elg_options);
     struct aws_event_loop *event_loop = aws_event_loop_group_get_next_loop(el_group);
 
     ASSERT_NOT_NULL(event_loop, "Event loop creation failed with error: %s", aws_error_debug_str(aws_last_error()));
