@@ -74,11 +74,15 @@ static struct aws_s3_platform_info s_c5n_9xlarge_platform_info = {
 /****** End c5n.9large *****/
 
 /***** Begin p4d.24xlarge and p4de.24xlarge ****/
-static struct aws_byte_cursor s_p4d_socket1_array[] = {AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth0"),
-                                                       AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth1")};
+static struct aws_byte_cursor s_p4d_socket1_array[] = {
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth0"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth1"),
+};
 
-static struct aws_byte_cursor s_p4d_socket2_array[] = {AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth2"),
-                                                       AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth3")};
+static struct aws_byte_cursor s_p4d_socket2_array[] = {
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth2"),
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("eth3"),
+};
 
 static struct aws_s3_cpu_group_info s_p4d_cpu_group_info_array[] = {
     {
@@ -289,9 +293,6 @@ static void s_destroy_loader(void *arg) {
     aws_hash_table_clean_up(&loader->lock_data.compute_platform_info_table);
     aws_mutex_clean_up(&loader->lock_data.lock);
 
-    /* clean up the memory we allocated in init() */
-    aws_mem_release(loader->allocator, loader->lock_data.current_env_platform_info.cpu_group_info_array);
-
     if (loader->lock_data.detected_instance_type) {
         aws_string_destroy(loader->lock_data.detected_instance_type);
     }
@@ -310,21 +311,12 @@ struct aws_s3_platform_info_loader *aws_s3_platform_info_loader_new(struct aws_a
     aws_mutex_init(&loader->lock_data.lock);
     aws_ref_count_init(&loader->ref_count, loader, s_destroy_loader);
 
-    /* we won't know an instance type, possibly ever, but it will be set if available before returning to the user. */
-    loader->lock_data.current_env_platform_info.has_recommended_configuration = false;
-    loader->lock_data.current_env_platform_info.cpu_group_info_array_length =
-        aws_system_environment_get_cpu_group_count(loader->current_env);
-    loader->lock_data.current_env_platform_info.cpu_group_info_array = aws_mem_calloc(
-        allocator,
-        loader->lock_data.current_env_platform_info.cpu_group_info_array_length,
-        sizeof(struct aws_s3_cpu_group_info));
-
-    for (size_t i = 0; i < loader->lock_data.current_env_platform_info.cpu_group_info_array_length; ++i) {
-        struct aws_s3_cpu_group_info *group_info = &loader->lock_data.current_env_platform_info.cpu_group_info_array[i];
-        group_info->cpu_group = (uint16_t)i;
-        group_info->cpus_in_group = aws_get_cpu_count_for_group((uint16_t)i);
-        /* when we have the ability to detect NIC affinity add that here. */
-    }
+    /* TODO: Implement runtime CPU information retrieval from the system. Currently, Valgrind detects a memory leak
+     * associated with the g_numa_node_of_cpu_ptr function (see: https://github.com/numactl/numactl/issues/3). This
+     * issue was addressed in version v2.0.13 of libnuma (see: https://github.com/numactl/numactl/pull/43). However,
+     * Amazon Linux 2 defaults to libnuma version v2.0.9, which lacks this fix. We need to suppress this
+     * warning as a false positive in older versions of libnuma. In the future, however, we will probably eliminate the
+     * use of numactl altogether. */
 
     AWS_FATAL_ASSERT(
         !aws_hash_table_init(
@@ -494,7 +486,7 @@ tear_down:
     return callback_info.instance_type;
 }
 
-struct aws_byte_cursor aws_s3_get_ec2_instance_type(struct aws_s3_platform_info_loader *loader) {
+struct aws_byte_cursor aws_s3_get_ec2_instance_type(struct aws_s3_platform_info_loader *loader, bool cached_only) {
     aws_mutex_lock(&loader->lock_data.lock);
     struct aws_byte_cursor return_cur;
     AWS_ZERO_STRUCT(return_cur);
@@ -507,6 +499,14 @@ struct aws_byte_cursor aws_s3_get_ec2_instance_type(struct aws_s3_platform_info_
             aws_string_bytes(loader->lock_data.detected_instance_type));
         goto return_instance_and_unlock;
     }
+    if (cached_only) {
+        AWS_LOGF_TRACE(
+            AWS_LS_S3_CLIENT,
+            "id=%p: Instance type has not been cached. Returning without trying to determine instance type since "
+            "cached_only is set.",
+            (void *)loader);
+        goto return_instance_and_unlock;
+    }
 
     AWS_LOGF_TRACE(
         AWS_LS_S3_CLIENT,
@@ -564,7 +564,7 @@ return_instance_and_unlock:
 const struct aws_s3_platform_info *aws_s3_get_platform_info_for_current_environment(
     struct aws_s3_platform_info_loader *loader) {
     /* getting the instance type will set it on the loader the first time if it can */
-    aws_s3_get_ec2_instance_type(loader);
+    aws_s3_get_ec2_instance_type(loader, false /*cached_only*/);
     /* will never be mutated after the above call. */
     return &loader->lock_data.current_env_platform_info;
 }
