@@ -10,6 +10,7 @@
 #include "aws/s3/private/s3_request.h"
 #include <aws/auth/credentials.h>
 #include <aws/common/clock.h>
+#include <aws/common/encoding.h>
 #include <aws/common/string.h>
 #include <aws/common/xml_parser.h>
 #include <aws/http/request_response.h>
@@ -42,20 +43,10 @@ const struct aws_byte_cursor g_trailer_header_name = AWS_BYTE_CUR_INIT_FROM_STRI
 const struct aws_byte_cursor g_request_validation_mode = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-mode");
 const struct aws_byte_cursor g_enabled = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("enabled");
 
-const struct aws_byte_cursor g_create_mpu_checksum_header_name =
+const struct aws_byte_cursor g_checksum_algorithm_header_name =
     AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-algorithm");
-const struct aws_byte_cursor g_crc32c_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32c");
-const struct aws_byte_cursor g_crc32_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-crc32");
-const struct aws_byte_cursor g_sha1_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha1");
-const struct aws_byte_cursor g_sha256_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-checksum-sha256");
-const struct aws_byte_cursor g_crc32c_create_mpu_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("CRC32C");
-const struct aws_byte_cursor g_crc32_create_mpu_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("CRC32");
-const struct aws_byte_cursor g_sha1_create_mpu_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("SHA1");
-const struct aws_byte_cursor g_sha256_create_mpu_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("SHA256");
-const struct aws_byte_cursor g_crc32c_complete_mpu_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ChecksumCRC32C");
-const struct aws_byte_cursor g_crc32_complete_mpu_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ChecksumCRC32");
-const struct aws_byte_cursor g_sha1_complete_mpu_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ChecksumSHA1");
-const struct aws_byte_cursor g_sha256_complete_mpu_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("ChecksumSHA256");
+const struct aws_byte_cursor g_sdk_checksum_algorithm_header_name =
+    AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-sdk-checksum-algorithm");
 const struct aws_byte_cursor g_accept_ranges_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("accept-ranges");
 const struct aws_byte_cursor g_acl_header_name = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("x-amz-acl");
 const struct aws_byte_cursor g_mp_parts_count_header_name =
@@ -686,6 +677,11 @@ int aws_s3_crt_error_code_from_recoverable_server_error_code_string(struct aws_b
     if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "RequestTimeTooSkewed")) {
         return AWS_ERROR_S3_REQUEST_TIME_TOO_SKEWED;
     }
+
+    if (aws_byte_cursor_eq_c_str_ignore_case(&error_code_string, "RequestTimeout")) {
+        return AWS_ERROR_S3_REQUEST_TIMEOUT;
+    }
+
     return AWS_ERROR_UNKNOWN;
 }
 
@@ -709,3 +705,60 @@ void aws_s3_request_finish_up_metrics_synced(struct aws_s3_request *request, str
         request->send_data.metrics = aws_s3_request_metrics_release(metrics);
     }
 }
+
+int aws_s3_check_headers_for_checksum(
+    struct aws_s3_meta_request *meta_request,
+    const struct aws_http_headers *headers,
+    struct aws_s3_checksum **out_checksum,
+    struct aws_byte_buf *out_checksum_buffer,
+    bool meta_request_level) {
+    AWS_PRECONDITION(meta_request);
+    AWS_PRECONDITION(out_checksum);
+    AWS_PRECONDITION(out_checksum_buffer);
+
+    if (!headers || aws_http_headers_count(headers) == 0) {
+        *out_checksum = NULL;
+        return AWS_OP_SUCCESS;
+    }
+    if (meta_request_level && aws_http_headers_has(headers, g_mp_parts_count_header_name)) {
+        /* g_mp_parts_count_header_name indicates it's a object was uploaded as a
+         * multipart upload. So, the checksum should not be applied to the meta request level.
+         * But we we want to check it for the request level. */
+        *out_checksum = NULL;
+        return AWS_OP_SUCCESS;
+    }
+
+    for (size_t i = 0; i < AWS_ARRAY_SIZE(s_checksum_algo_priority_list); i++) {
+        enum aws_s3_checksum_algorithm algorithm = s_checksum_algo_priority_list[i];
+        if (!aws_s3_meta_request_checksum_config_has_algorithm(meta_request, algorithm)) {
+            /* If user doesn't select this algorithm, skip */
+            continue;
+        }
+        const struct aws_byte_cursor algorithm_header_name =
+            aws_get_http_header_name_from_checksum_algorithm(algorithm);
+        struct aws_byte_cursor checksum_value;
+        if (aws_http_headers_get(headers, algorithm_header_name, &checksum_value) == AWS_OP_SUCCESS) {
+            /* Found the checksum header, keep the header value and initialize the running checksum */
+            size_t encoded_len = 0;
+            aws_base64_compute_encoded_len(aws_get_digest_size_from_checksum_algorithm(algorithm), &encoded_len);
+            if (checksum_value.len == encoded_len - 1) {
+                /* encoded_len includes the nullptr length. -1 is the expected length. */
+                aws_byte_buf_init_copy_from_cursor(out_checksum_buffer, meta_request->allocator, checksum_value);
+                *out_checksum = aws_checksum_new(meta_request->allocator, algorithm);
+                if (!*out_checksum) {
+                    AWS_LOGF_ERROR(
+                        AWS_LS_S3_META_REQUEST,
+                        "Could not create checksum for algorithm: %d, due to error code %d (%s)",
+                        algorithm,
+                        aws_last_error_or_unknown(),
+                        aws_error_str(aws_last_error_or_unknown()));
+                    return AWS_OP_ERR;
+                }
+                return AWS_OP_SUCCESS;
+            }
+            break;
+        }
+    }
+    *out_checksum = NULL;
+    return AWS_OP_SUCCESS;
+}
