@@ -66,7 +66,8 @@
 #include "../fipsmodule/rsa/internal.h"
 
 #ifdef ENABLE_DILITHIUM
-#include "../dilithium/sig_dilithium.h"
+#include "../dilithium/ml_dsa.h"
+#include "../dilithium/internal.h"
 #endif
 
 
@@ -121,12 +122,11 @@ static int bn_print(BIO *bp, const char *name, const BIGNUM *num, int off) {
   // TODO(davidben): Do we need to do this? We already print "(Negative)" above
   // and negative values are never valid in keys anyway.
   size_t len = BN_num_bytes(num);
-  uint8_t *buf = OPENSSL_malloc(len + 1);
+  uint8_t *buf = OPENSSL_zalloc(len + 1);
   if (buf == NULL) {
     return 0;
   }
 
-  buf[0] = 0;
   BN_bn2bin(num, buf + 1);
   int ret;
   if (len > 0 && (buf[1] & 0x80) != 0) {
@@ -314,9 +314,9 @@ static int eckey_priv_print(BIO *bp, const EVP_PKEY *pkey, int indent) {
 
 #ifdef ENABLE_DILITHIUM
 
-// Dilithium keys.
+// MLDSA keys.
 
-static int do_dilithium3_print(BIO *bp, const EVP_PKEY *pkey, int off, int ptype) {
+static int do_mldsa_65_print(BIO *bp, const EVP_PKEY *pkey, int off, int ptype) {
   if (pkey == NULL) {
     OPENSSL_PUT_ERROR(EVP, ERR_R_PASSED_NULL_PARAMETER);
     return 0;
@@ -326,21 +326,21 @@ static int do_dilithium3_print(BIO *bp, const EVP_PKEY *pkey, int off, int ptype
     return 0;
   }
 
-  const DILITHIUM3_KEY *key = pkey->pkey.ptr;
+  const PQDSA *pqdsa = pkey->pkey.pqdsa_key->pqdsa;
   int bit_len = 0;
 
   if (ptype == 2) {
-    bit_len = DILITHIUM3_PRIVATE_KEY_BYTES;
+    bit_len = pqdsa->private_key_len;
     if (BIO_printf(bp, "Private-Key: (%d bit)\n", bit_len) <= 0) {
       return 0;
     }
-    print_hex(bp, key->priv, bit_len, off);
+    print_hex(bp, pkey->pkey.pqdsa_key->private_key, bit_len, off);
   } else {
-    bit_len = DILITHIUM3_PUBLIC_KEY_BYTES;
+    bit_len = pqdsa->public_key_len;
     if (BIO_printf(bp, "Public-Key: (%d bit)\n", bit_len) <= 0) {
       return 0;
     }
-    int ret = print_hex(bp, key->pub, bit_len, off);
+    int ret = print_hex(bp, pkey->pkey.pqdsa_key->public_key, bit_len, off);
     if (!ret) {
       return 0;
     }
@@ -349,12 +349,12 @@ static int do_dilithium3_print(BIO *bp, const EVP_PKEY *pkey, int off, int ptype
   return 1;
 }
 
-static int dilithium3_pub_print(BIO *bp, const EVP_PKEY *pkey, int indent) {
-  return do_dilithium3_print(bp, pkey, indent, 1);
+static int mldsa_65_pub_print(BIO *bp, const EVP_PKEY *pkey, int indent) {
+  return do_mldsa_65_print(bp, pkey, indent, 1);
 }
 
-static int dilithium3_priv_print(BIO *bp, const EVP_PKEY *pkey, int indent) {
-  return do_dilithium3_print(bp, pkey, indent, 2);
+static int mldsa_65_priv_print(BIO *bp, const EVP_PKEY *pkey, int indent) {
+  return do_mldsa_65_print(bp, pkey, indent, 2);
 }
 
 #endif
@@ -387,9 +387,9 @@ static EVP_PKEY_PRINT_METHOD kPrintMethods[] = {
     },
 #ifdef ENABLE_DILITHIUM
     {
-        EVP_PKEY_DILITHIUM3,
-        dilithium3_pub_print,
-        dilithium3_priv_print,
+        EVP_PKEY_PQDSA,
+        mldsa_65_pub_print,
+        mldsa_65_priv_print,
         NULL /* param_print */,
     },
 #endif
