@@ -119,8 +119,12 @@ OPENSSL_INLINE int CRYPTO_is_SHAEXT_capable(void) {
   return (OPENSSL_ia32cap_get()[2] & (1 << 29)) != 0;
 }
 
+// AVX512VL  | AVX512BW | AVX512DQ | AVX512F
+// 1u << 31  | 1u << 30 | 1u << 17 | 1u << 16
+// 1100_0000_0000_0011_0000_0000_0000_0000
+#define CPU_CAP_AVX512_BITFLAGS 0xC0030000
 OPENSSL_INLINE int CRYPTO_is_AVX512_capable(void) {
-  return (OPENSSL_ia32cap_get()[2] & 0xC0030000) == 0xC0030000;
+  return (OPENSSL_ia32cap_get()[2] & CPU_CAP_AVX512_BITFLAGS) == CPU_CAP_AVX512_BITFLAGS;
 }
 
 OPENSSL_INLINE int CRYPTO_is_VAES_capable(void) {
@@ -131,8 +135,17 @@ OPENSSL_INLINE int CRYPTO_is_VPCLMULQDQ_capable(void) {
   return (OPENSSL_ia32cap_get()[3] & (1u << (42 - 32))) != 0;
 }
 
+// AVX512VL  | AVX512BW | AVX512_IFMA | AVX512DQ | AVX512F
+// 1u << 31  | 1u << 30 | 1u << 21    | 1u << 17 | 1u << 16
+// 1100_0000_0010_0011_0000_0000_0000_0000
+#define CPU_CAP_AVX512IFMA_BITFLAGS 0xC0230000
 OPENSSL_INLINE int CRYPTO_is_AVX512IFMA_capable(void) {
-    return (OPENSSL_ia32cap_get()[3] & (1u << 31 | 1u << 21 |1u << 17 | 1u << 16)) != 0;
+#if defined(OPENSSL_WINDOWS)
+  return 0;
+#else
+  return (OPENSSL_ia32cap_get()[2] & CPU_CAP_AVX512IFMA_BITFLAGS) ==
+         CPU_CAP_AVX512IFMA_BITFLAGS;
+#endif
 }
 
 OPENSSL_INLINE int CRYPTO_is_VBMI2_capable(void) {
@@ -230,21 +243,61 @@ OPENSSL_INLINE int CRYPTO_is_ARMv8_GCM_8x_capable(void) {
   return ((OPENSSL_armcap_P & ARMV8_SHA3) != 0 &&
           ((OPENSSL_armcap_P & ARMV8_NEOVERSE_V1) != 0 ||
            (OPENSSL_armcap_P & ARMV8_NEOVERSE_V2) != 0 ||
-           (OPENSSL_armcap_P & ARMV8_APPLE_M1) != 0));
+           (OPENSSL_armcap_P & ARMV8_APPLE_M) != 0));
 }
 
 OPENSSL_INLINE int CRYPTO_is_ARMv8_wide_multiplier_capable(void) {
   return (OPENSSL_armcap_P & ARMV8_NEOVERSE_V1) != 0 ||
            (OPENSSL_armcap_P & ARMV8_NEOVERSE_V2) != 0 ||
-           (OPENSSL_armcap_P & ARMV8_APPLE_M1) != 0;
+           (OPENSSL_armcap_P & ARMV8_APPLE_M) != 0;
 }
 
 OPENSSL_INLINE int CRYPTO_is_ARMv8_DIT_capable(void) {
-  return (OPENSSL_armcap_P & ARMV8_DIT) != 0;
+  return (OPENSSL_armcap_P & (ARMV8_DIT | ARMV8_DIT_ALLOWED)) ==
+    (ARMV8_DIT | ARMV8_DIT_ALLOWED);
 }
 
+// This function is used only for testing; hence, not inlined
+OPENSSL_EXPORT int CRYPTO_is_ARMv8_DIT_capable_for_testing(void);
+
 #endif  // OPENSSL_ARM || OPENSSL_AARCH64
 
+#if defined(AARCH64_DIT_SUPPORTED)
+// (TODO): See if we can detect the DIT capability in Windows environment
+
+// armv8_get_dit gets the value of the DIT flag from the CPU.
+OPENSSL_EXPORT uint64_t armv8_get_dit(void);
+
+// armv8_set_dit sets the CPU DIT flag to 1 and returns its original value
+// before it was called.
+OPENSSL_EXPORT uint64_t armv8_set_dit(void);
+
+// armv8_restore_dit takes as input a value to restore the CPU DIT flag to.
+OPENSSL_EXPORT void armv8_restore_dit(volatile uint64_t *original_dit);
+
+#if defined(ENABLE_AUTO_SET_RESET_DIT)
+// SET_DIT_AUTO_RESET can be inserted in the caller's application at
+// the beginning of the code section that makes repeated calls to AWS-LC
+// functions. The flag will be automatically restored to its original value
+// at the end of the scope.
+// This can minimise the effect on performance of repeatedly setting and
+// disabling DIT.
+// Instead of the macro, the functions above can be used.
+// An example of their usage is present in the benchmarking function
+// `Speed()` in `tool/speed.cc` when the option `-dit` is passed in.
+#define SET_DIT_AUTO_RESET                      \
+  volatile uint64_t _dit_restore_orig                \
+         __attribute__((cleanup(armv8_restore_dit))) \
+          OPENSSL_UNUSED = armv8_set_dit();
+
+#else
+#define SET_DIT_AUTO_RESET
+#endif  // ENABLE_AUTO_SET_RESET_DIT
+
+#else
+#define SET_DIT_AUTO_RESET
+#endif  // AARCH64_DIT_SUPPORTED
+
 #if defined(OPENSSL_PPC64LE)
 
 // CRYPTO_is_PPC64LE_vcrypto_capable returns true iff the current CPU supports
